/*
 * Fabric3
 * Copyright (c) 2009-2012 Metaform Systems
 *
 * Fabric3 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version, with the
 * following exception:
 *
 * Linking this software statically or dynamically with other
 * modules is making a combined work based on this software.
 * Thus, the terms and conditions of the GNU General Public
 * License cover the whole combination.
 *
 * As a special exception, the copyright holders of this software
 * give you permission to link this software with independent
 * modules to produce an executable, regardless of the license
 * terms of these independent modules, and to copy and distribute
 * the resulting executable under terms of your choice, provided
 * that you also meet, for each linked independent module, the
 * terms and conditions of the license of that module. An
 * independent module is a module which is not derived from or
 * based on this software. If you modify this software, you may
 * extend this exception to your version of the software, but
 * you are not obligated to do so. If you do not wish to do so,
 * delete this exception statement from your version.
 *
 * Fabric3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the
 * GNU General Public License along with Fabric3.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------
 *
 * Portions originally based on Apache Tuscany 2007
 * licensed under the Apache 2.0 license.
 *
 */
package org.fabric3.implementation.pojo.injection;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.List;

import org.fabric3.spi.objectfactory.ObjectCreationException;
import org.fabric3.spi.objectfactory.ObjectFactory;

/**
 * Resolves targets configured in a multiplicity by delegating to object factories and returning an <code>Array</code> containing object instances
 *
 * @version $Rev: 11083 $ $Date: 2012-06-21 10:38:50 +0000 (Thu, 21 Jun 2012) $
 */
public class ArrayMultiplicityObjectFactory implements MultiplicityObjectFactory<Object> {
    private List<ObjectFactory<?>> factories = new ArrayList<ObjectFactory<?>>();
    private Class interfaceType;
    private boolean cleared = true;

    public ArrayMultiplicityObjectFactory(Class interfaceType) {
        assert interfaceType != null : "Interface type was null";
        assert factories != null : "Object factories were null";
        this.interfaceType = interfaceType;
    }

    public Object getInstance() throws ObjectCreationException {
        Object array = Array.newInstance(interfaceType, factories.size());
        for (int i = 0; i < factories.size(); i++) {
            Array.set(array, i, factories.get(i).getInstance());
        }
        return array;
    }

    public void addObjectFactory(ObjectFactory<?> objectFactory, Object key) {
        if (!cleared) {
            clear();
        }
        factories.add(objectFactory);
    }

    public void clear() {
        factories.clear();
        cleared = true;
    }

    public void startUpdate() {
        cleared = false;
    }

    public void endUpdate() {
        cleared = true;
    }
}
